@@ -1,4 +1,21 @@
-/*
+/*  Copyright 1996   Grant R. Guenther,  based on work of Itai Nahshon
+ *   http://www.torque.net/ziptool.html
+ *  Copyright 1997-1999,2001,2002,2005,2007,2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * scsi.c
  * Iomega Zip/Jaz drive tool
  * change protection mode and eject disk
@@ -53,6 +70,10 @@ struct scsi_ioctl_command {
 #include <camlib.h>
 #endif
 
+#if defined(OS_netbsd) || defined(OS_netbsdelf)
+#include <sys/scsiio.h>
+#endif
+
 int scsi_max_length(void)
 {
 #ifdef OS_linux
@@ -73,9 +94,9 @@ int scsi_open(const char *name, int flag, int mode, void **extra_data)
     else
         return -1;
 #else
-    return open(name, O_RDONLY | O_LARGEFILE |
+    return open(name, O_RDONLY | O_LARGEFILE | O_BINARY
 #ifdef O_NDELAY
-		O_NDELAY
+		| O_NDELAY
 #endif
 	/* O_RDONLY  | dev->mode*/);
 #endif
@@ -148,31 +169,31 @@ int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
 	return 0;
 	
 #elif defined OS_linux
-	struct scsi_ioctl_command scsi_cmd;
+	struct scsi_ioctl_command my_scsi_cmd;
 
 
-	memcpy(scsi_cmd.cmd, cdb, cmdlen);        /* copy command */
+	memcpy(my_scsi_cmd.cmd, cdb, cmdlen);        /* copy command */
 
 	switch (mode) {
 		case SCSI_IO_READ:
-			scsi_cmd.inlen = 0;
-			scsi_cmd.outlen = len;
+			my_scsi_cmd.inlen = 0;
+			my_scsi_cmd.outlen = len;
 			break;
 		case SCSI_IO_WRITE:
-			scsi_cmd.inlen = len;
-			scsi_cmd.outlen = 0;
-			memcpy(scsi_cmd.cmd + cmdlen,data,len);
+			my_scsi_cmd.inlen = len;
+			my_scsi_cmd.outlen = 0;
+			memcpy(my_scsi_cmd.cmd + cmdlen,data,len);
 			break;
 	}
 	
-	if (ioctl(fd, SCSI_IOCTL_SEND_COMMAND, &scsi_cmd) < 0) {
+	if (ioctl(fd, SCSI_IOCTL_SEND_COMMAND, &my_scsi_cmd) < 0) {
 		perror("scsi_io");
 		return -1;
 	}
 	
 	switch (mode) {
 		case SCSI_IO_READ:
-			memcpy(data, &scsi_cmd.cmd[0], len);
+			memcpy(data, &my_scsi_cmd.cmd[0], len);
 			break;
 		case SCSI_IO_WRITE:
 			break;
@@ -181,52 +202,52 @@ int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
 	return 0;  /* where to get scsi status? */
 
 #elif (defined _SCO_DS) && (defined SCSIUSERCMD)
-	struct scsicmd scsi_cmd;
+	struct scsicmd my_scsi_cmd;
 
-	memset(scsi_cmd.cdb, 0, SCSICMDLEN);	/* ensure zero pad */
-	memcpy(scsi_cmd.cdb, cdb, cmdlen);
-	scsi_cmd.cdb_len = cmdlen;
-	scsi_cmd.data_len = len;
-	scsi_cmd.data_ptr = data;
-	scsi_cmd.is_write = mode == SCSI_IO_WRITE;
-	if (ioctl(fd,SCSIUSERCMD,&scsi_cmd) == -1) {
+	memset(my_scsi_cmd.cdb, 0, SCSICMDLEN);	/* ensure zero pad */
+	memcpy(my_scsi_cmd.cdb, cdb, cmdlen);
+	my_scsi_cmd.cdb_len = cmdlen;
+	my_scsi_cmd.data_len = len;
+	my_scsi_cmd.data_ptr = data;
+	my_scsi_cmd.is_write = mode == SCSI_IO_WRITE;
+	if (ioctl(fd,SCSIUSERCMD,&my_scsi_cmd) == -1) {
 		perror("scsi_io: SCSIUSERCMD");
 		return -1;
 	}
-	if (scsi_cmd.host_sts != 0 || scsi_cmd.target_sts != 0) {
+	if (my_scsi_cmd.host_sts != 0 || my_scsi_cmd.target_sts != 0) {
 		fprintf(stderr, "scsi_io: scsi status: host=%x; target=%x\n",
-		(unsigned)scsi_cmd.host_sts,(unsigned)scsi_cmd.target_sts);
+		(unsigned)my_scsi_cmd.host_sts,(unsigned)my_scsi_cmd.target_sts);
 		return -1;
 	}
 	return 0;
 #elif defined sgi
- 	struct dsreq scsi_cmd;
+ 	struct dsreq my_scsi_cmd;
 
-	scsi_cmd.ds_cmdbuf = (char *)cdb;
-	scsi_cmd.ds_cmdlen = cmdlen;
-	scsi_cmd.ds_databuf = data;
-	scsi_cmd.ds_datalen = len;
+	my_scsi_cmd.ds_cmdbuf = (char *)cdb;
+	my_scsi_cmd.ds_cmdlen = cmdlen;
+	my_scsi_cmd.ds_databuf = data;
+	my_scsi_cmd.ds_datalen = len;
        	switch (mode) {
 	case SCSI_IO_READ:
-	  scsi_cmd.ds_flags = DSRQ_READ|DSRQ_SENSE;
+	  my_scsi_cmd.ds_flags = DSRQ_READ|DSRQ_SENSE;
 	  break;
 	case SCSI_IO_WRITE:
-	  scsi_cmd.ds_flags = DSRQ_WRITE|DSRQ_SENSE;
+	  my_scsi_cmd.ds_flags = DSRQ_WRITE|DSRQ_SENSE;
 	  break;
         } 
-	scsi_cmd.ds_time = 10000;
-	scsi_cmd.ds_link = 0;
-	scsi_cmd.ds_synch =0;
-	scsi_cmd.ds_ret =0;
-	if (ioctl(fd, DS_ENTER, &scsi_cmd) == -1) {
+	my_scsi_cmd.ds_time = 10000;
+	my_scsi_cmd.ds_link = 0;
+	my_scsi_cmd.ds_synch =0;
+	my_scsi_cmd.ds_ret =0;
+	if (ioctl(fd, DS_ENTER, &my_scsi_cmd) == -1) {
                 perror("scsi_io");
                 return -1;
         }
 
-        if(scsi_cmd.ds_status) {
+        if(my_scsi_cmd.ds_status) {
                 errno = 0;
                 fprintf(stderr,"scsi status=%x\n",  
-                        (unsigned short)scsi_cmd.ds_status);
+                        (unsigned short)my_scsi_cmd.ds_status);
                 return -1;
         }
         
@@ -270,6 +291,38 @@ int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
 	  return -1;
       }
       return 0;
+#elif defined(OS_netbsd) || defined(OS_netbsdelf)
+ 	struct scsireq sc;
+
+	memset(&sc, 0, sizeof(sc));
+	memcpy(sc.cmd, cdb, cmdlen);
+	sc.cmdlen = cmdlen;
+	sc.databuf = data;
+	sc.datalen = len;
+	sc.senselen = 0;
+	sc.timeout = 10000;
+	switch (mode) {
+	case SCSI_IO_READ:
+	  sc.flags = SCCMD_READ;
+	  break;
+	case SCSI_IO_WRITE:
+	  sc.flags = SCCMD_WRITE;
+	  break;
+	}
+
+	if (ioctl(fd, SCIOCCOMMAND, &sc) == -1) {
+                perror("SCIOCCOMMAND ioctl");
+                return -1;
+	}
+
+	if (sc.retsts) {
+                errno = EIO;
+                fprintf(stderr, "SCSI command failed, retsts %d\n", 
+sc.retsts);
+                return -1;
+	}
+
+        return 0;
 #else
       fprintf(stderr, "scsi_io not implemented\n");
       return -1;
